package com.bocloud.cmp.controller;

import com.bocloud.cmp.domain.CloudVendorBean;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.service.CloudVendorService;
import com.bocloud.cmp.service.SyncService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 云供应商Controller
 *
 * @author DZG
 * @since V1.0 2016年9月1日
 */
@Tag(name = "云供应商")
@RestController
@RequestMapping("/v1/vendors")
public class CloudVendorController {

    @Autowired
    private CloudVendorService cloudVendorService;
    @Autowired
    @Qualifier("defaultSyncService")
    private SyncService syncService;

    /**
     * 获取云供应商列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "获取云供应商列表")
    @GetMapping
    public GeneralResult<GridBean<CloudVendor>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.list(pager, context);
    }

    /**
     * 添加云供应商
     *
     * @param vendor
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建云供应商")
    @PostMapping
    public GeneralResult<CloudVendor> create(@RequestBody CloudVendor vendor,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.create(vendor, context);
    }

    /**
     * 修改云供应商
     *
     * @param id
     * @param bean
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改云供应商信息")
    @PutMapping("/{id}")
    public GeneralResult<CloudVendor> modify(@PathVariable(value = Common.ID) Long id,
        @RequestBody CloudVendorBean bean, @Value(Common.REQ_CONTEXT) RequestContext context) {
        bean.setId(id);
        return cloudVendorService.modify(bean, context);
    }

    /**
     * 移除云供应商
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "移除云供应商信息")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.remove(id, context);
    }

    /**
     * 查看云供应商详细信息
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看云供应商详细信息")
    @GetMapping("/{id}")
    public GeneralResult<CloudVendor> detail(@PathVariable(value = Common.ID) Long id) {
        return cloudVendorService.detail(id);
    }

    /**
     * 冻结云供应商
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "云平台操作")
    @PatchMapping("/{id}")
    public GeneralResult action(@PathVariable(value = Common.ID) Long id,
        @RequestParam(value = Common.ACTION, required = false) String action,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.action(id, action, context);
    }

    /**
     * 更新认证信息
     *
     * @param id
     * @param cloudVendor
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改云平台密码")
    @PatchMapping("/{id}/password")
    public GeneralResult updatePassword(@PathVariable(value = Common.ID) Long id, @RequestBody CloudVendor cloudVendor,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.updatePassword(id, cloudVendor, context);
    }

    /**
     * 修改云平台超分比
     *
     * @param id
     * @param cloudVendor
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "修改云平台超分比")
    @PatchMapping("/{id}/excessRatio")
    public GeneralResult updateExcessRatio(@PathVariable(value = Common.ID) Long id,
        @RequestBody CloudVendor cloudVendor, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.updateExcessRatio(id, cloudVendor, context);
    }

    /**
     * 获取云供应商列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式获取云平台列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.condition(condition, context);
    }

    /**
     * 同步云供应商数据
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "同步云供应商数据")
    @PostMapping("/{id}/sync")
    public GeneralResult sync(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return syncService.sync(id, context, false);
    }

    /**
     * 批量开启/关闭告警
     *
     * @param ids
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "批量开启/关闭告警")
    @PatchMapping("/alarm")
    public GeneralResult batchOperationAlarm(@RequestBody List<Long> ids,
        @RequestParam(value = Common.ACTION) String action, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.batchOperationAlarm(ids, action, context.getTarget());
    }

    /**
     * 同步平台指定类型数据
     *
     * @param id
     * @param context
     * @return
     */
    // @Operation(tags = { "CMC", "CSC" }, summary = "同步平台指定类型数据")
    // @PostMapping("/{id}/sync/{syncType}")
    // public GeneralResult sync(@PathVariable(value = Common.ID) Long id,
    // @PathVariable(value = "syncType") String syncType, @Value(Common.REQ_CONTEXT) RequestContext context) {
    // try {
    // SyncType type = SyncType.valueOf(syncType);
    // return syncService.sync(id, context, type);
    // } catch (Exception e) {
    // return new GeneralResult(false, "不支持的数据类型：" + syncType + " " + e, "400", "修改数据类型");
    // }
    // }

    /**
     * 查看云供应商资源统计信息
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看云供应商资源容量信息")
    @GetMapping("/{id}/summary")
    public GeneralResult summary(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorService.summary(id, context);
    }

}
