package com.bocloud.cmp.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.LabelParam;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.DataStore;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.model.DataStoreBean;
import com.bocloud.cmp.service.DataStoreService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@Tag(name = "数据存储")
@RestController
@RequestMapping("/v1/datastores")
public class DataStoreController {

    @Autowired
    private DataStoreService dataStoreService;

    /**
     * 查看数据存储列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询数据存储列表")
    @GetMapping
    public GeneralResult<GridBean<DataStore>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return dataStoreService.list(pager, context);
    }

    /**
     * 查看数据存储列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式查询数据存储列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return dataStoreService.condition(condition, context);
    }

    /**
     * 添加数据存储
     *
     * @param dataStore
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建数据存储")
    @PostMapping
    public GeneralResult<DataStore> create(@RequestBody DataStore dataStore, @Value(Common.REQ_CONTEXT) RequestContext context) {
        dataStore.setCreatorId(context.getTarget());
        dataStore.setMenderId(context.getTarget());
        return dataStoreService.create(dataStore, context);
    }

    /**
     * 修改数据存储
     *
     * @param id
     * @param bean
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "更新数据存储信息")
    @PutMapping("/{id}")
    public GeneralResult<DataStore> modify(@PathVariable(value = Common.ID) Long id, @RequestBody DataStoreBean bean,
                                           @Value(Common.REQ_CONTEXT) RequestContext context) {
        bean.setId(id);
        return dataStoreService.modify(bean, context.getTarget());
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除数据存储存储")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return dataStoreService.remove(id, context);
    }

    /**
     * 详细信息
     *
     * @param id 任务ID
     * @return 操作结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询数据存储详情")
    @GetMapping("/{id}")
    public GeneralResult<DataStore> detail(@PathVariable(value = Common.ID) Long id) {
        return dataStoreService.detail(id);
    }

    /**
     * 查看云主机列表
     *
     * @param id
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询数据存储列表")
    @GetMapping("/{id}/vms")
    public GeneralResult<GridBean<CloudServer>> servers(@PathVariable(value = Common.ID) Long id, Pager pager) {
        return dataStoreService.servers(pager, id);
    }

    /**
     * 查看宿主机列表
     *
     * @param id
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询主机列表")
    @GetMapping("/{id}/hosts")
    public GeneralResult<GridBean<Server>> hosts(@PathVariable(value = Common.ID) Long id, Pager pager) {
        return dataStoreService.hosts(pager, id);
    }

    /**
     * 卸载数据存储
     *
     * @param id
     * @param hostId
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "卸载数据存储")
    @DeleteMapping("/{id}/hosts")
    public GeneralResult unmount(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "hostId") Long hostId,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return dataStoreService.unmount(id, hostId, context.getTarget());
    }

    /**
     * 批量配置数据存储标签
     *
     * @param param
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "批量配置数据存储标签")
    @PutMapping("/tags")
    public GeneralResult batchModifyLabel(@RequestBody LabelParam param,
                                          @Value(Common.REQ_CONTEXT) RequestContext context) {
        String params = null;
        if (null != param) {
            params = JSONObject.toJSONString(param);
        }
        return dataStoreService.batchModifyLabel(params, context);
    }

}
