package com.bocloud.cmp.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.Image;
import com.bocloud.cmp.entity.Volume;
import com.bocloud.cmp.model.BatchUpdateImageParam;
import com.bocloud.cmp.model.ImgBean;
import com.bocloud.cmp.service.ImageService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.List;

@RestController
@RequestMapping("/v1/images")
@Tag(name = "镜像管理")
@Slf4j
public class ImageController {

    @Autowired
    private ImageService imageService;
    private static final String IMAGE_SERVICE = "imageService";
    @Autowired
    private ApplicationContext applicationContext;

    /**
     * 添加
     *
     * @param bean
     * @param type
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建镜像")
    @PostMapping
    public GeneralResult create(@RequestBody Image bean,
                                @RequestParam(value = Common.TYPE, required = false) String type,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        type = StringUtils.isBlank(type) ? bean.getType() : type;
        ImageService dispatchService = applicationContext
                .getBean((type == null ? "" : type.toLowerCase()) + IMAGE_SERVICE, ImageService.class);
        return dispatchService.create(bean, context);
    }

    /**
     * 查看镜像列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询镜像列表")
    @GetMapping
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return imageService.list(pager, context);
    }

    /**
     * 查看镜像列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式查询镜像列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return imageService.condition(condition, context);
    }

    /**
     * 获取当前nfs路径下所有的iso文件
     *
     * @return
     */
    @GetMapping(value = "/iso")
    @Operation(tags = {"CMC", "CSC"}, summary = "查找nfs上的镜像")
    public GeneralResult isoList(@RequestParam(value = Common.TYPE, required = false) String type) {
        ImageService dispatchService = applicationContext
                .getBean((type == null ? "" : type.toLowerCase()) + IMAGE_SERVICE, ImageService.class);
        return dispatchService.isoList();
    }

    /**
     * 查看镜像详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看镜像详情")
    @GetMapping("/{id}")
    public GeneralResult<Image> detail(@PathVariable(value = Common.ID) Long id) {
        return imageService.detail(id);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除信息")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = Common.TYPE, required = false) String type,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        ImageService dispatchService = applicationContext
                .getBean((type == null ? "" : type.toLowerCase()) + IMAGE_SERVICE, ImageService.class);
        return dispatchService.remove(id, context);
    }

    /**
     * 修改镜像
     *
     * @param id
     * @param bean
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改镜像信息")
    @PutMapping("/{id}")
    public GeneralResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestBody ImgBean bean, @Value(Common.REQ_CONTEXT) RequestContext context) {
        bean.setId(id);
        return imageService.modify(bean, context.getTarget());
    }

    /**
     * 更新镜像
     *
     * @param imgBean
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "更新镜像信息")
    @PatchMapping("/{id}")
    public GeneralResult patch(@PathVariable(value = Common.ID) Long id, @RequestBody ImgBean imgBean,
                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null == imgBean) {
            return new GeneralResult<>(false, "参数格式错误");
        }
        String type = imgBean.getType();
        ImageService dispatchService = applicationContext
                .getBean((type == null ? "" : type.toLowerCase()) + IMAGE_SERVICE, ImageService.class);
        JSONObject object = JSONTools.isJSONObj(JSON.toJSONString(imgBean));
        if (null != object) {
            return dispatchService.patch(id, object, context, imgBean.getAction());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 批量删除镜像
     *
     * @param ids
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "镜像批量删除")
    @DeleteMapping
    public GeneralResult batchRemove(@RequestBody List<Long> ids, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return imageService.batchRemove(ids, context);
    }

    /**
     * 批量更新镜像
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "批量更新镜像信息")
    @PutMapping
    public GeneralResult batchUpdate(@RequestBody BatchUpdateImageParam params,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return imageService.batchUpdate(params, context);
    }

    /**
     * 下载镜像
     *
     * @param id
     * @param request
     * @param response
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "下载镜像")
    @GetMapping("/{id}/files")
    public void download(@PathVariable(value = Common.ID) Long id, HttpServletRequest request,
                         HttpServletResponse response) {
        GeneralResult result = imageService.download(id);
        if (result.isSuccess()) {
            InputStream fin = null;
            ServletOutputStream out = null;
            File file = null;
            try {
                log.info("镜像文件路径：" + result.getData());
                file = new File((String) result.getData());
                if (!file.exists()) {
                    response.setCharacterEncoding("UTF-8");
                    response.getWriter().println(JSONObject.toJSON(result));
                    response.flushBuffer();
                    return;
                }
                fin = new FileInputStream(file);
                out = response.getOutputStream();
                response.setCharacterEncoding("utf-8");
                String fileName = file.getName();
                response.setContentType("application/octet-stream");
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "utf-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setHeader("Content-Disposition", "attachment; fileName=" + fileName);
                IOUtils.copy(fin, out);
            } catch (Exception e) {
                log.error("Get error message：", e);
                response.setCharacterEncoding("UTF-8");
                try {
                    response.getWriter().println(JSONObject.toJSON(result));
                    response.flushBuffer();
                } catch (IOException e1) {
                    log.error("Get error message:{}", e1.getMessage());
                }
            } finally {
                try {
                    if (fin != null) {
                        fin.close();
                    }
                    if (out != null) {
                        out.close();
                    }
                    if (file != null) {
                        boolean flag = file.delete();
                        log.info("临时镜像文件删除状态：{}", flag);
                    }
                } catch (IOException e) {
                    log.error("Get error message:{}", e.getMessage());
                }

            }
        }
    }

    /**
     * 查询镜像使用的存储卷
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询镜像使用的存储卷")
    @GetMapping("/{id}/volumes")
    public GeneralResult<List<Volume>> listVolumeByImage(@PathVariable(value = Common.ID) Long id) {
        return imageService.listVolumeByImage(id);
    }

    /**
     * 查询镜像使用的云主机
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询使用该镜像的云主机")
    @GetMapping("/{id}/servers")
    public GeneralResult<GridBean<CloudServer>> listServersByImage(Pager pager, @PathVariable(value = Common.ID) Long id) {
        return imageService.listServerByImage(id, pager);
    }

}
