package com.bocloud.cmp.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.Ip;
import com.bocloud.cmp.model.IpPatchParams;
import com.bocloud.cmp.model.IpScanParam;
import com.bocloud.cmp.service.IpService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.Optional;

@RestController
@RequestMapping("/v1/ips")
@Tag(name = "IP管理")
public class IpController {

    @Autowired
    private IpService ipService;

    /**
     * 查看IP列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询IP列表")
    @GetMapping
    public GeneralResult<GridBean<Ip>> list(Pager pager) {
        pager = Optional.ofNullable(pager).orElse(new Pager());
        return ipService.list(pager);
    }

    /**
     * 根据condition查询IP列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "根据condition查询IP列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipService.condition(condition, context);
    }

    /**
     * 查看IP详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询IP详细数据")
    @GetMapping("/{id}")
    public GeneralResult<Ip> detail(@PathVariable(value = Common.ID) Long id) {
        return ipService.detail(id);
    }

    /**
     * 删除IP
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除IP地址")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id) {
        return ipService.remove(id);
    }

    /**
     * 批量保存IP
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "批量保存IP")
    @PostMapping
    public GeneralResult batchSave(@RequestParam(value = Common.PARAMS, required = true) String params,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONObject.parseObject(params);
        if (null == object) {
            return ResultTools.formatErrResult();
        }
        return ipService.batchSave(object, context);
    }

    /**
     * IP地址池扫描
     *
     * @param param
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "IP地址池扫描")
    @PutMapping
    public GeneralResult scan(@RequestBody IpScanParam param, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipService.scan(param, context);
    }

    /**
     * IP操作
     *
     * @param id
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "IP地址池扫描")
    @PatchMapping("/{id}")
    public GeneralResult patch(@PathVariable(value = Common.ID) Long id,
                               @RequestBody IpPatchParams params,
                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipService.action(id, params.getAction(), params.getRemark(), context);
    }


    /**
     * IP批量操作
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "IP批量操作")
    @PatchMapping
    public GeneralResult batchUpdate(@RequestBody IpPatchParams params, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipService.batchUpdate(params.getIds(), params.getAction(), params.getRemark(), context);
    }

    /**
     * 根据subnet删除ip,pms服务调用
     *
     * @param params
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "根据subnet删除ip")
    @DeleteMapping("/subnets")
    public GeneralResult deleteBySubnetId(@RequestParam(value = Common.PARAMS) String params) {
        JSONObject object = JSONObject.parseObject(params);
        if (null == object) {
            return ResultTools.formatErrResult();
        }
        String ip = object.getString("ip");
        Long subnetId = object.getLong("subnetId");
        return ipService.deleteBySubnetId(ip, subnetId);
    }
}
