package com.bocloud.cmp.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.LabelParam;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.service.ServerService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 物理服务器相关接口 前端请求的时候加入type参数，取值为： HOST CloudProvider INSTALLATION
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1")
@Tag(name = "物理服务器管理")
public class ServerController {

    @Autowired
    private ApplicationContext applicationContext;

    private static final String SERVER_SERVICE = "ServerService";

    /**
     * 获取服务器列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "获取物理服务器列表")
    @GetMapping("/hosts")
    public GeneralResult<GridBean> list(Pager pager, @RequestParam(value = Common.TYPE, required = false) String type) {

        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);

        return serverService.list(pager);
    }

    /**
     * 根据condition获取物理服务器列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "根据condition获取物理服务器列表")
    @GetMapping("/hosts/condition")
    public GeneralResult condition(Pager pager,
                                   @RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @RequestParam(value = Common.TYPE, required = false) String type, HttpServletRequest request,
                                   HttpServletResponse response) {

        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);

        if (StringUtils.isNotEmpty(condition)) {
            JSONObject conditionObj = JSONObject.parseObject(condition);
            if ("queryByIp".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.queryByIp(conditionObj.getString("ip"));
            }
            if ("listByCluster".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.listByCluster(conditionObj.getLong("clusterId"));
            }
            if ("listByClusters".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.listByClusters(conditionObj.getJSONArray("clusterIds").toJavaList(Long.class));
            }
            if ("listByDefaultCluster".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.listByDefaultCluster(conditionObj.getLong("vendorId"));
            }
            if ("listByCondition".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.list(conditionObj.toJSONString());
            }
            if ("listByServerConfigs".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.listByServerConfigs(conditionObj.getJSONArray("serverIds"));
            }
            if ("listByVendorType".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.listByVendorType(conditionObj.toJSONString());
            }
        }
        return new GeneralResult(false, "查询失败");
    }

    /**
     * 查看物理主机详细信息
     *
     * @param id      物理主机ID
     * @param context 操作者
     * @return 操作结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看物理主机详情")
    @GetMapping("/hosts/{id}")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = Common.TYPE, required = false) String type,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {

        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);

        return serverService.detail(id);
    }

    /**
     * 查看物理主机基本信息
     *
     * @param id      物理主机ID
     * @param context 操作者
     * @return 操作结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看物理主机基本信息")
    @RequestMapping(value = "/hosts/{id}/basic", method = {RequestMethod.GET})
    public GeneralResult<Server> basic(@PathVariable(value = Common.ID) Long id,
                                       @RequestParam(value = Common.TYPE, required = false) String type,
                                       @Value(Common.REQ_CONTEXT) RequestContext context) {

        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);

        return serverService.basic(id);
    }

    @Operation(tags = {"CMC", "CSC"}, summary = "物理主机移入机柜")
    @RequestMapping(value = "/server/{id}/inRack", method = {RequestMethod.POST})
    public GeneralResult inRack(@RequestParam(value = Common.RACK, required = true) Long rackId,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ResultTools.noImplementedResult();
    }

    /**
     * 获取监控数据接口
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "获取监控数据")
    @RequestMapping(value = "/hosts/{id}/metrics", method = {RequestMethod.GET})
    public GeneralResult performance(@RequestParam(value = Common.PARAMS, required = false) String params,
                                     @RequestParam(value = Common.TYPE, required = false) String type,
                                     @Value(Common.REQ_CONTEXT) RequestContext context, @PathVariable(value = Common.ID) Long id) {

        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        return serverService.performance(params, id, context);
    }

    /**
     * 更新物理机的资源分组
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "更新资源分组")
    @RequestMapping(value = "/hosts/groups", method = {RequestMethod.PUT})
    public GeneralResult<String> updateResourceGroup(
            @RequestParam(value = Common.PARAMS, required = false) String params,
            @RequestParam(value = Common.TYPE, required = false) String type,
            @Value(Common.REQ_CONTEXT) RequestContext context, @PathVariable(value = Common.ID) Long id) {
        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            String idStr = object.getString("id");
            String resourceGroup = object.getString("resourceGroup");
            if (StringUtils.isNotEmpty(idStr)) {
                String[] ids = idStr.split(",");
                return serverService.updateResourceGroup(ids, resourceGroup, context.getTarget());
            }
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 查询指定集群下的宿主机
     *
     * @param listByCluster
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询指定集群下的宿主机")
    @RequestMapping(value = "/clusters/{id}/hosts", method = {RequestMethod.GET})
    public GeneralResult<List<Server>> listByCluster(@PathVariable(value = Common.ID) Long listByCluster,
                                                     @RequestParam(value = Common.TYPE, required = false) String type,
                                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        return serverService.listByCluster(listByCluster);
    }

    /**
     * 查询云平台下的宿主机
     *
     * @param params
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询云平台下的宿主机")
    @RequestMapping(value = "/vendors/{id}/hosts", method = {RequestMethod.GET})
    public GeneralResult<List<Server>> listByVendor(
            @RequestParam(value = Common.PARAMS, required = false) String params,
            @RequestParam(value = Common.TYPE, required = false) String type,
            @PathVariable(value = Common.ID) Long id) {
        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            return serverService.listByVendor(id);
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 更新物理主机状态
     *
     * @param params
     * @param uuid
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "更新物理主机状态")
    @RequestMapping(value = "/server/{uuid}/uuid/status/update", method = {RequestMethod.POST})
    public GeneralResult updateStatus(@RequestParam(value = Common.PARAMS, required = true) String params,
                                      @RequestParam(value = Common.TYPE, required = false) String type,
                                      @PathVariable(value = Common.UUID) String uuid) {
        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        return serverService.updateStatus(params, uuid);
    }

    /**
     * 批量配置宿主机标签
     *
     * @param param
     * @param type
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "批量配置宿主机标签")
    @PutMapping("/hosts/tags")
    public GeneralResult<String> batchModifyLabel(@RequestBody(required = false) LabelParam param,
                                                  @RequestParam(value = Common.TYPE, required = false) String type,
                                                  @Value(Common.REQ_CONTEXT) RequestContext context) {
        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        String params = null;
        if (null != param) {
            params = JSONObject.toJSONString(param);
        }
        return serverService.batchModifyLabel(params, context);
    }

    /**
     * 查看未管理的powervc云主机
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看未管理的powervc云主机")
    @RequestMapping(value = "/hosts/{id}/vms", method = {RequestMethod.GET})
    public GeneralResult<List<CloudServer>> listUnmanagedPowervm(@PathVariable(value = Common.ID) Long id, Pager pager,
                                                                 @RequestParam(value = Common.CONDITION, required = false) String condition,
                                                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        if (StringUtils.isNotEmpty(condition)) {
            JSONObject conditionObj = JSONObject.parseObject(condition);
            if ("listUnmanagedPowervm".equalsIgnoreCase(conditionObj.getString("condition"))) {
                return serverService.listUnmanagedPowervm(pager.getPage(), pager.getRows(), pager.getParams(),
                        pager.getSorter(), pager.getSimple());
            }
            if ("listvmByvhost".equalsIgnoreCase(conditionObj.getString("condition"))) {
                Long venderId = conditionObj.getJSONObject("param").getLong(Common.VENDORID);
                String hostname = conditionObj.getJSONObject("param").getString("hostname");
                return serverService.listvmByvhost(venderId, hostname);
            }
        }
        return new GeneralResult(false, "未知类型");
    }

    /**
     * powervc取消管理物理机上的云主机
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "powervc取消管理云主机")
    @RequestMapping(value = "/hosts/{id}/vms", method = {RequestMethod.DELETE})
    public GeneralResult unmanagePowervm(@RequestParam(value = Common.PARAMS, required = false) String params,
                                         @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        ServerService serverService = applicationContext.getBean(SERVER_SERVICE, ServerService.class);
        JSONObject jsonObject = JSONTools.isJSONObj(params);
        return ResultTools.formatErrResult();
    }

}
