package com.bocloud.cmp.internal;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.TenantPool;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * SOC租户内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月4日
 */
@Component
@Slf4j
public class SmsTenantService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;
	private static Service service = Service.create("sms");
	private Map<String, Object> headers = new HashMap<>(4);

	public GeneralResult<Tenant> detail(Long id) {
		RemoteService remote = serviceFactory.build(service, "/v1/tenants/" + id, HttpMethod.GET, headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, Tenant.class), "success");
	}

	public List<Tenant> listTenantsByIds(List<Long> ids) {
		ArrayList<Param> params=null;
		if (null!=ids && !ids.isEmpty()){
			String idString = JSONArray.toJSONString(ids);
			params = Lists.newArrayList(new Param(MapTools.simpleMap("id", idString.substring(1,idString.length()-1)), Sign.IN));
		}
		Map<String, Object> paramMap = MapTools.simpleMap("simple", false);
		paramMap.put("page", 1);
		paramMap.put("rows", Integer.MAX_VALUE);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants" , HttpMethod.GET, headers, paramMap);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new ArrayList<>();
		}
		String data = JSONObject.toJSONString(result.getData());
		return JSONObject.parseObject(data).getJSONArray("rows").toJavaList(Tenant.class);
	}

	public GeneralResult checkPoolGroup(Long vendorId) {
		Map<String, Object> params = Maps.newHashMap();
		JSONObject object = new JSONObject();
		object.put("condition", "checkPoolGroup");
		object.put("vendorId", vendorId);
		params.put(Common.CONDITION, object);
		RemoteService remote = serviceFactory.build(service, "/v1/tenants", HttpMethod.GET, headers, params);
		return remote.invoke();
	}

	public GeneralResult<List<Tenant>> listChildren(Long parentId) {
		Map<String, Object> params = Maps.newHashMap();
		params.put("parentId", parentId);
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants/" + parentId + "/children",
				HttpMethod.GET, headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONArray.parseArray(data, Tenant.class), "success");
	}

	public GeneralResult<List<Tenant>> listAll(List<Param> paramList) {
		Map<String, Object> params = Maps.newHashMap();
		params.put(Common.PAGE, 1);
		params.put(Common.ROWS, Integer.MAX_VALUE);
		params.put(Common.SIMPLE, false);
		params.put(Common.PARAMS, JSONObject.toJSONString(paramList));
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants", HttpMethod.GET, headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		GridBean gridBean = JSONObject.parseObject(data, GridBean.class);
		return new GeneralResult<>(true,
				JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Tenant.class), "success");
	}

	/**
	 * 根据租户ID查询租户资源池
	 *
	 * @param tenantId
	 * @return
	 */
	public List<TenantPool> listPoolByTenant(Long tenantId) {
		Map<String, Object> params = Maps.newHashMap();
		params.put("tenantId", tenantId);
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants/" + tenantId + "/pools",
				HttpMethod.GET, headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new ArrayList<>();
		}
		String data = JSONObject.toJSONString(result.getData());
		return JSONArray.parseArray(data, TenantPool.class);
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
		this.headers.put("source", currentService.getService().getName());
	}

	public GeneralResult<List<Tenant>> getListTenantTree(Long parentId) {
		Map<String, Object> params = Maps.newHashMap();
		params.put("parentId", parentId);
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants/tree",
				HttpMethod.GET, headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONArray.parseArray(data, Tenant.class), "success");
	}

	/**
	 * 根据资源池ID查询租户资源池
	 *
	 * @param poolGroupId
	 * @return
	 */
	public List<TenantPool> listPoolByPoolsId(Long poolGroupId) throws Exception {
		Map<String, Object> params = Maps.newHashMap();
		params.put("poolGroupId", poolGroupId);
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants/" + poolGroupId + "/poolsId",
				HttpMethod.GET, headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			throw new Exception(result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return JSONArray.parseArray(data, TenantPool.class);
	}

	/**
	 * 根据资源池ID删除资源池与租户关联
	 *
	 * @param poolId
	 * @return
	 */
	public GeneralResult deleteTenantPools(Long poolId) {
		Map<String, Object> params = Maps.newHashMap();
		params.put("poolId", poolId);
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants/pools",
				HttpMethod.DELETE, headers, params);
		GeneralResult result = remoteService.invoke();
		return result;
	}

	public GeneralResult queryParentTenant(Long id) {
		Map<String, Object> params = Maps.newHashMap();
		params.put("id", id);
		RemoteService remoteService = serviceFactory.build(service, "/v1/tenants/highest/tenant/"+id, HttpMethod.GET, headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		return new GeneralResult(true, result.getData(), "查询成功");
	}

	public Tenant parentTenant(Long tenantId) {
		try {
			Map<String, Object> paramMap = new HashMap<String, Object>();
			RemoteService remoteService = serviceFactory.build(service, "/v1/tenants/" + tenantId + "/root", HttpMethod.GET, headers, paramMap);
			GeneralResult result = remoteService.invoke();
			if (result.isFailed()) {
				return null;
			}
			String data = JSONObject.toJSONString(result.getData());
			return JSONObject.parseObject(data, Tenant.class);
		} catch (Exception e) {
			log.error("根据租户ID查询一级租户异常：" ,e);
			return null;
		}
	}
}
