package com.bocloud.cmp.service.impl;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.NotifyMessage;
import com.bocloud.cmp.domain.RackDevice;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.entity.ServerAssets;
import com.bocloud.cmp.entity.ServerConfig;
import com.bocloud.cmp.repository.ServerAssetsRepository;
import com.bocloud.cmp.repository.ServerConfigRepository;
import com.bocloud.cmp.repository.ServerRepository;
import com.bocloud.cmp.service.ServerService;
import com.bocloud.cmp.util.ResourceEventPublisher;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ResultTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Service("hostServerService")
@Slf4j
public class HostServerServiceImpl extends ServerServiceImpl implements ServerService {

    @Autowired
    private ServerRepository serverRepository;
    @Autowired
    private ServerConfigRepository serverConfigRepository;
    @Autowired
    private ServerAssetsRepository serverAssetsRepository;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    @Qualifier("ServerService")
    private ServerService serverService;

    @Override
    public GeneralResult<String> create(String params, RequestContext context, Boolean isImport, String instance) {
    	return ResultTools.noImplementedResult();
    }

    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult modify(String params, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    /**
     * 根据uuid修改物理主机
     *
     * @author sxp
     * @since Mar 7, 2018
     * @since 2018-03-08 modify by sxp 对于错误uuid或历史遗留数据无需处理，并视为正常情况
     */
    @Override
    public GeneralResult modifyByUuid(String params, String uuid, RequestContext context) {
        try {
            if (StringUtils.isEmpty(uuid)) {
                return new GeneralResult(false, "修改物理主机失败，原因：uuid的值为空");
            }
            Long id = serverRepository.queryIdByUuid(uuid);
            if (id == null) {
                return new GeneralResult(true, "系统中不存在uuid为[" + uuid + "]的物理主机信息，无需修改");
            }

            // begin--追加server,server_config,server_assets的id属性
            JSONObject json = JSONTools.isJSONObj(params);
            if (null == json) {
                return new GeneralResult(false, "根据uuid修改物理主机时数据不能为空!");
            }

            String serverStr = json.getString("server");
            String serverConfigStr = json.getString("serverConfig");
            String serverAssetsStr = json.getString("serverAssets");

            Server server = JSONObject.parseObject(serverStr, Server.class);
            ServerConfig config = JSONObject.parseObject(serverConfigStr, ServerConfig.class);
            ServerAssets assets = JSONObject.parseObject(serverAssetsStr, ServerAssets.class);

            if (server != null) {
                server.setId(id);
            }
            if (config != null) {
                config.setServerId(id);
                ServerConfig configOfDb = serverConfigRepository.getBySid(id);
                if (configOfDb != null) {
                    config.setId(configOfDb.getId());
                }
            }
            if (assets != null) {
                assets.setServerId(id);
                ServerAssets assetsOfDb = serverAssetsRepository.getBySid(id);
                if (assetsOfDb != null) {
                    assets.setId(assetsOfDb.getId());
                }
            }

            json.put("server", server);
            json.put("serverConfig", config);
            json.put("serverAssets", assets);
            params = json.toJSONString();
            // end----追加server,server_config,server_assets的id属性
            return modify(params, context);
        } catch (Exception e) {
            log.error("修改物理主机失败," + e.getMessage(), e);
            return new GeneralResult(false, "修改物理主机失败");
        }
    }

    @Override
    public GeneralResult inRack(RackDevice rackDevice, RequestContext context) {
        String path = Server.class.getSimpleName() + "_" + rackDevice.getDeviceId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            this.serverRepository.belong(rackDevice, context.getTarget());
            return new GeneralResult(true, "添加成功");
        } catch (Exception e) {
            log.error("Assign rack to server fail:", e);
            return new GeneralResult(false, "添加失败");
        }
    }

    @Override
    public GeneralResult outRack(Long id, RequestContext context) {
        String path = Server.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            this.serverRepository.release(id, context.getTarget());
            return new GeneralResult(true, "移除成功");
        } catch (Exception e) {
            log.error("Free server from rack fail:", e);
            return new GeneralResult(false, "移除失败");
        }
    }

    @Override
    public GeneralResult freePMs() {
        List<Server> list = null;
        try {
            list = this.serverRepository.listFree();
        } catch (Exception e) {
            log.error("Query free server list fail:", e);
            return new GeneralResult(false, "");
        }
        return new GeneralResult(true, list, "查询成功");
    }

    private void sendMessage(boolean success, NotifyMessage message, String content) {
        // 站内信和邮件发送
        amqpTemplate.convertAndSend("event.notice.queue", message);
        // 界面消息
        OperateResult operateResult = new OperateResult(success, content, "platform.server", null,
                OperateResult.OperateCategory.ResourceEvent, null);
        resourceEventPublisher.send(operateResult);
    }


    @Override
    public GeneralResult start(Long id, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult stop(Long id, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult restart(Long id, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult batchStart(String params, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult batchStop(String params, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult batchRemove(String params, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    /**
     * 更新物理主机状态
     *
     * @param params 参数
     * @param uuid   物理主机的uuid
     * @return 操作结果
     * @author sxp
     * @since Mar 12, 2018
     */
    @Override
    public GeneralResult updateStatus(String params, String uuid) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<Map<String, Object>> download(Map<String, Object> paramsMap) {
    	return ResultTools.noImplementedResult();
    }

    /**
     * 下载日志文件
     *
     * @param paramsMap
     * @return
     */
    private GeneralResult<Map<String, Object>> downloadLogFile(Map<String, Object> paramsMap) {
    	return ResultTools.noImplementedResult();
    }

    /**
     * 下载excel模板
     *
     * @param paramsMap
     * @return
     */
    private GeneralResult<Map<String, Object>> downloadExcel(Map<String, Object> paramsMap) {
    	return ResultTools.noImplementedResult();
    }

    /**
     * 批量导入主机资源
     */
    @Override
    public GeneralResult upload(Map<String, Object> paramsMap, MultipartFile excel, Long contextId, String instance)
            throws IOException {
    	return ResultTools.noImplementedResult();
    }

    /**
     * 修改机房机柜
     *
     * @param id
     * @param rackId
     * @param context
     * @return
     */
    @Override
    public GeneralResult patch(Long id, Long rackId, RequestContext context) {
        String path = Server.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Server server = this.serverRepository.query(id);
            if (null == server) {
                log.warn("Server data does not exist!");
                return new GeneralResult(false, "数据不存在");
            }
            server.setRackId(rackId);
            server.setMenderId(context.getTarget());
            this.serverRepository.update(server);
            return new GeneralResult(true, "修改成功");
        } catch (Exception e) {
            log.error("Modify server fail:", e);
            return new GeneralResult(false, "修改失败");
        }
    }
}
