package com.bocloud.cmp.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.ServerLabelParam;
import com.bocloud.cmp.entity.*;
import com.bocloud.cmp.enums.ResourceCategory;
import com.bocloud.cmp.repository.*;
import com.bocloud.cmp.service.LabelService;
import com.bocloud.cmp.util.ResourceEventPublisher;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Service("labelService")
@Slf4j
public class LabelServiceImpl implements LabelService {

    @Autowired
    private LabelRepository labelRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private ResourceEventRepository resourceEventRepository;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private VpcRepository vpcRepository;
    @Autowired
    private FloatingIpRepository floatingIpRepository;
    @Autowired
    private ElasticIpRepository elasticIpRepository;
    @Autowired
    private VolumeRepository volumeRepository;

    @Override
    public GeneralResult<GridBean<Label>> list(Pager pager, RequestContext context) {
        List<Label> list = null;
        int total = 0;
        GridBean gridBean = null;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
            Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
            Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
            total = labelRepository.count(params);
            if (simple) {
                List<Label> beans = this.labelRepository.list(params);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = labelRepository.list(page, rows, params, sorter);
                gridBean = GridHelper.getBean(page, rows, total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("list label failure:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Label> detail(Long id) {
        try {
            Label label = labelRepository.query(id);
            return new GeneralResult<>(true, label, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public GeneralResult<Label> create(Label label) {
        String path = Label.class.getSimpleName() + "_create";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "请求超时");
            if (labelRepository.exists(label.getName(), label.getContent(), label.getCategory())) {
                return new GeneralResult<>(false, "标签已存在");
            }
            labelRepository.save(label);
            return new GeneralResult<>(true, label, "添加成功");
        } catch (Exception e) {
            log.error("Create label error:", e);
            return new GeneralResult<>(false, "添加失败");
        }
    }

    @Override
    public GeneralResult<Label> modify(Label label) {
        String path = Label.class.getSimpleName() + "_" + label.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Label oldLabel = this.labelRepository.query(label.getId());
            if (null == oldLabel) {
                log.warn("Label does not exist!");
                return new GeneralResult<>(false, "数据不存在");
            }
            if (!label.getName().equals(oldLabel.getName())) {
                if (labelRepository.exists(label.getName())) {
                    return new GeneralResult<>(false, "名称已存在");
                }
            }
            oldLabel.setContent(label.getContent());
            oldLabel.setName(label.getName());
            oldLabel.setCategory(label.getCategory());
            this.labelRepository.update(oldLabel);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("Modify label error:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult remove(Long id) {
        String path = Label.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Label oldLabel = this.labelRepository.query(id);
            if (null == oldLabel) {
                log.warn("Label does not exist!");
                return new GeneralResult<>(false, "数据不存在");
            }
            this.labelRepository.remove(id);
            // 删除已经绑定的资源标签
            // processResource(oldLabel);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove label error:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult createResources(ServerLabelParam serverLabelParam, RequestContext context) {
        try {
            if (null == serverLabelParam) {
                return new GeneralResult<>(false, "参数格式错误");
            }
            if (null == serverLabelParam.getVendorId() || 0L == serverLabelParam.getVendorId()) {
                return new GeneralResult<>(false, "请传入平台信息");
            }
            new Thread(() -> {
                String operate = ".serverlabel";
                Long vendorId = serverLabelParam.getVendorId();
                try {
                    List<Long> targetIds = serverLabelParam.getTargetIds();
                    Long targetId = serverLabelParam.getTargetId();
                    if (null != targetId) {
                        targetIds.add(targetId);
                        targetIds.stream().distinct();
                    }
                    //操作平台
                    CloudVendor vendor = cloudVendorRepository.query(vendorId);
                    Assert.notNull(vendor, "云平台不存在");
                    String resourceLabel = serverLabelParam.getResourceLabel();
                    ResourceEvent event = null;
                    if (!ListTool.isEmpty(targetIds)) {
                        for (Long target : targetIds) {
                            switch (serverLabelParam.getType()) {
                                case Computer:
                                    CloudServer cloudServer = cloudServerRepository.query(target);
                                    if (null == cloudServer) {
                                        log.error("云主机ID [{}] 不存在", target);
                                        continue;
                                    }
                                    cloudServer.setResourceLabel(resourceLabel);
                                    cloudServerRepository.update(cloudServer);
                                    operate = ResourceCategory.Computer.name();
                                    event = new ResourceEvent(targetId, operate + "标签配置", operate + "配置标签成功：" + resourceLabel, context.getTarget(),
                                            new Date(), operate, vendorId, context.getCatalog());
                                    break;
                                case Vpc:
                                    Vpc vpc = vpcRepository.query(target);
                                    if (null == vpc) {
                                        log.error("Vpc ID [{}] 不存在", target);
                                        continue;
                                    }
                                    vpc.setResourceLabel(resourceLabel);
                                    vpcRepository.update(vpc);
                                    operate = ResourceCategory.Vpc.name();
                                    event = new ResourceEvent(targetId, operate + "标签配置", operate + "配置标签成功：" + resourceLabel, context.getTarget(),
                                            new Date(), operate, vendorId, context.getCatalog());
                                    break;
                                case Storage:
                                    Volume volume = volumeRepository.query(target);
                                    if (null == volume) {
                                        log.error("Volume ID [{}] 不存在", target);
                                        continue;
                                    }
                                    volume.setResourceLabel(resourceLabel);
                                    volumeRepository.update(volume);
                                    operate = ResourceCategory.Storage.name();
                                    event = new ResourceEvent(targetId, operate + "标签配置", operate + "配置标签成功：" + resourceLabel, context.getTarget(),
                                            new Date(), operate, vendorId, context.getCatalog());
                                    break;
                                case FloatingIp:
                                case FLOATING_IP:
                                    // 根据平台获取浮动IP
                                    if ("ALIYUN".equalsIgnoreCase(vendor.getType()) || "HUAWEI".equalsIgnoreCase(vendor.getType()) || "HCSO".equalsIgnoreCase(vendor.getType())) {
                                        ElasticIp elasticIp = elasticIpRepository.query(target);
                                        if (null == elasticIp) {
                                            log.error("ElasticIp ID [{}] 不存在", target);
                                            continue;
                                        }
                                        elasticIp.setResourceLabel(resourceLabel);
                                        elasticIpRepository.update(elasticIp);
                                    } else {
                                        FloatingIp floatingIp = floatingIpRepository.query(target);
                                        if (null == floatingIp) {
                                            log.error("FloatingIp ID [{}] 不存在", target);
                                            continue;
                                        }
                                        floatingIp.setResourceLabel(resourceLabel);
                                        floatingIpRepository.update(floatingIp);
                                    }
                                    operate = ResourceCategory.FloatingIp.name();
                                    event = new ResourceEvent(targetId, operate + "标签配置", operate + "配置标签成功：" + resourceLabel, context.getTarget(),
                                            new Date(), operate, vendorId, context.getCatalog());
                                    break;
                                default:
                                    operate = serverLabelParam.getType().name();
                                    resourceEventPublisher.send(new OperateResult(false, "标签操作失败：不存在此类型的标签", operate,
                                            MapTools.simpleMap(Common.VENDORID, vendorId), OperateResult.OperateCategory.ResourceEvent,
                                            context));
                                    break;
                            }
                            if (null != event) {
                                resourceEventRepository.save(event);
                            }
                            operate = operate + ".serverlabel";
                            resourceEventPublisher
                                    .send(new OperateResult(true, "标签操作成功", operate, MapTools.simpleMap(Common.VENDORID, vendorId),
                                            OperateResult.OperateCategory.ResourceEvent, context));
                        }
                    }
                } catch (Exception e) {
                    resourceEventPublisher.send(new OperateResult(false, "标签操作失败：" + e.getMessage(), operate,
                            MapTools.simpleMap(Common.VENDORID, vendorId), OperateResult.OperateCategory.ResourceEvent,
                            context));
                }
            }).start();
            return new GeneralResult<String>(true, "任务已下发，正在执行...");
        } catch (Exception e) {
            log.error("资源标签操作异常：" + e.getMessage());
            return new GeneralResult<>(false, "资源标签操作异常：" + e.getMessage());
        }
    }

    @Override
    public GeneralResult isExistsValues(List<JSONObject> parseArray) {
        try {
            for (int i = parseArray.size() - 1; i >= 0; i--) {
                JSONObject jsonObject = parseArray.get(i);
                String key = jsonObject.getString("key");
                String value = jsonObject.getString("value");
                Label label = labelRepository.getLabelByName(key);
                if (null != label) {
                    List<String> contentArray = JSON.parseArray(label.getContent(), String.class);
                    if (null != contentArray && !contentArray.contains(value)) {
                        parseArray.remove(jsonObject);
                    }
                } else {
                    parseArray.remove(jsonObject);
                }
            }
            return new GeneralResult(true, parseArray, "标签检验成功");
        } catch (Exception e) {
            log.error("标签校验异常：" + e.getMessage());
            return new GeneralResult<>(false, "标签校验异常：" + e.getMessage());
        }
    }
}
