package com.bocloud.cmp.service.impl;

import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Random;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.NameRule;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.internal.SmsUserService;
import com.bocloud.cmp.repository.NameRuleRepository;
import com.bocloud.cmp.service.NameRuleService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import com.google.common.collect.Maps;

import lombok.extern.slf4j.Slf4j;

/**
 * 命名规则service实现
 *
 * @author wangyu
 * @version 3.0.0-SNAPSHOT
 * @since 2018/7/2
 */
@Service("nameRuleService")
@Slf4j
public class NameRuleServiceImpl implements NameRuleService {

	@Autowired
	private NameRuleRepository nameRuleRepository;
	@Autowired
	private LockFactory lockFactory;
	@Autowired
	private SmsTenantService smsTenantService;
	@Autowired
	private SmsUserService smsUserService;
	@Autowired
	private SmsProjectService smsProjectService;

	@Override
	public GeneralResult<GridBean<NameRule>> list(Pager pager, RequestContext context) {
		List<NameRule> list = null;
		int total = 0;
		GridBean gridBean = null;
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);

			if (null != context.getTenant() && 0L != context.getTenant()) {
				GeneralResult<User> result = smsUserService.detail(context.getTarget());
				if (result.isFailed()) {
					return new GeneralResult<>(false, "登录用户存在问题!");
				}
				User loginUser = result.getData();
				if (null == loginUser) {
					return new GeneralResult<>(false, "登录用户存在问题!");
				}
				params.add(new Param(MapTools.simpleMap(Common.TENANTID, loginUser.getTenantId()), Sign.EQ));
			} else {
				params.add(new Param(MapTools.simpleMap(Common.TENANTID, 0L), Sign.EQ));
			}
			total = nameRuleRepository.count(params);
			if (simple) {
				List<NameRule> beans = this.nameRuleRepository.list(params);
				gridBean = new GridBean(1, 1, total, beans);
			} else {
				list = nameRuleRepository.list(page, rows, params, sorter);
				gridBean = GridHelper.getBean(page, rows, total, list);
			}
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list name rule failure:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult<NameRule> detail(Long id) {
		try {
			NameRule nameRule = nameRuleRepository.query(id);
			return new GeneralResult<>(true, nameRule, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult<NameRule> create(NameRule nameRule, RequestContext context) {
		try {
			if (nameRuleRepository.exists(nameRule.getName(), context.getTenant())) {
				return new GeneralResult<>(false, "名称已存在");
			}
			if (nameRuleRepository.exists(nameRule.getHasTenant(), nameRule.getHasUser(), nameRule.getHasBusiness(),
					nameRule.getHasProject(), nameRule.getCustomize(), nameRule.getSuffixType(), context.getTenant())) {
				return new GeneralResult<>(false, "存在重复的命名规则");
			}
			if (null == nameRule.getCurrentNum()) {
				nameRule.setCurrentNum(0L);
			}
			nameRule.setTenantId(context.getTenant());
			nameRuleRepository.save(nameRule);
			return new GeneralResult<>(true, nameRule, "添加成功");
		} catch (Exception e) {
			log.error("Create name rule error:", e);
			return new GeneralResult<>(false, "添加失败");
		}
	}

	@Override
	public GeneralResult modify(NameRule nameRule, RequestContext context) {
		String path = NameRule.class.getSimpleName() + "_" + nameRule.getId();
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			NameRule oldRule = this.nameRuleRepository.query(nameRule.getId());
			if (null == oldRule) {
				log.warn("Name rule does not exist!");
				return new GeneralResult<>(false, "数据不存在");
			}
			if (!nameRule.getName().equals(oldRule.getName())) {
				if (nameRuleRepository.exists(nameRule.getName(), context.getTenant())) {
					return new GeneralResult<>(false, "名称已存在");
				}
			}
			oldRule.setName(nameRule.getName());
			if (nameRule.getHasTenant().equals(oldRule.getHasTenant())
					|| nameRule.getHasUser().equals(oldRule.getHasUser())
					|| nameRule.getHasBusiness().equals(oldRule.getHasBusiness())
					|| nameRule.getHasProject().equals(oldRule.getHasProject())
					|| !nameRule.getCustomize().equals(oldRule.getCustomize())
					|| !nameRule.getSuffixType().equals(oldRule.getSuffixType())) {
				if (nameRuleRepository.exists(nameRule.getHasTenant(), nameRule.getHasUser(), nameRule.getHasBusiness(),
						nameRule.getHasProject(), nameRule.getCustomize(), nameRule.getSuffixType(),
						context.getTenant())) {
					return new GeneralResult<>(false, "暂未做任何修改，请重新选择");
				}
				oldRule.setSuffixType(nameRule.getSuffixType());
				oldRule.setSuffixLength(nameRule.getSuffixLength());
				oldRule.setHasTenant(nameRule.getHasTenant());
				oldRule.setHasUser(nameRule.getHasUser());
				oldRule.setHasBusiness(nameRule.getHasBusiness());
				oldRule.setHasProject(nameRule.getHasProject());
				oldRule.setCustomize(nameRule.getCustomize());
			}
			this.nameRuleRepository.update(oldRule);
			return new GeneralResult<>(true, "修改成功");
		} catch (Exception e) {
			log.error("Modify name rule error:", e);
			return new GeneralResult<>(false, "修改失败");
		}
	}

	@Override
	public GeneralResult remove(Long id) {
		String path = NameRule.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			NameRule nameRule = this.nameRuleRepository.query(id);
			if (null == nameRule) {
				log.warn("Name rule does not exist!");
				return new GeneralResult<>(false, "数据不存在");
			}
			this.nameRuleRepository.remove(id);
			return new GeneralResult<>(true, "删除成功");
		} catch (Exception e) {
			log.error("Remove name rule error:", e);
			return new GeneralResult<>(false, "删除失败");
		}
	}

	@Override
	public GeneralResult<String> generate(Long id, JSONObject object) {
		try {
			NameRule nameRule = this.nameRuleRepository.query(id);
			if (null == nameRule) {
				log.warn("Name rule does not exist!");
				return new GeneralResult<>(false, "命名规则不存在");
			}
			Long tenantId = object.getLong("tenantId");
			Long userId = object.getLong("userId");
			Long projectId = object.getLong("projectId");
			String catalog = object.getString("catalog");
			String customize = nameRule.getCustomize();
			String name = "";
			if (null != tenantId && 0L != tenantId && null != nameRule.getHasTenant() && nameRule.getHasTenant()) {
				GeneralResult<Tenant> result = smsTenantService.detail(tenantId);
				if (result.isFailed()) {
					return new GeneralResult<>(false, "命名生成失败，租户不存在");
				}
				Tenant tenant = result.getData();
				if (null == tenant) {
					return new GeneralResult<>(false, "命名生成失败，租户不存在");
				}
				name += tenant.getAccount();
			}
			if (null != userId && 0L != userId && null != nameRule.getHasUser() && nameRule.getHasUser()) {
				if (catalog.equalsIgnoreCase(RequestContext.Catalog.User.name())
						|| catalog.equalsIgnoreCase(RequestContext.Catalog.Manager.name())) {
					GeneralResult<User> result = smsUserService.detail(userId);
					if (result.isFailed()) {
						return new GeneralResult<>(false, "命名生成失败，用户不存在");
					}
					User user = result.getData();
					if (null == user) {
						return new GeneralResult<>(false, "命名生成失败，用户不存在");
					}
					if (StringUtils.isEmpty(name)) {
						name += user.getAccount();
					} else {
						name += "_" + user.getAccount();
					}
				} else if (catalog.equalsIgnoreCase(RequestContext.Catalog.Tenant.name())) {
					GeneralResult<Tenant> result = smsTenantService.detail(userId);
					if (result.isFailed()) {
						return new GeneralResult<>(false, "命名生成失败，租户不存在");
					}
					Tenant tenant = result.getData();
					if (null == tenant) {
						return new GeneralResult<>(false, "命名生成失败，租户不存在");
					}
					name += tenant.getAccount();
				} else {
					return new GeneralResult<>(false, "未知用户角色");
				}
			}
			if (null != projectId && 0L != projectId) {
				GeneralResult<Project> result = smsProjectService.basic(projectId);
				if (result.isFailed()) {
					return new GeneralResult<>(false, "命名生成失败，项目不存在");
				}
				Project project = result.getData();
				if (null == project) {
					return new GeneralResult<>(false, "命名生成失败，项目不存在");
				}
				// 暂不需要业务
//                if (null != nameRule.getHasBusiness() && nameRule.getHasBusiness()) {
//                    GeneralResult<Business> businessResult = smsBusinessService.detail(project.getBusinessId());
//                    if (businessResult.isFailed()) {
//                        return new GeneralResult<>(false, "命名生成失败，业务不存在");
//                    }
//                    Business business = businessResult.getData();
//                    if (null == business) {
//                        return new GeneralResult<>(false, "命名生成失败，业务不存在");
//                    }
//                    if (StringUtils.isEmpty(name)) {
//                        name += business.getCode();
//                    } else {
//                        name += "_" + business.getCode();
//                    }
//                }
				if (null != nameRule.getHasProject() && nameRule.getHasProject()) {
					if (StringUtils.isEmpty(name)) {
						name += project.getName();
					} else {
						name += "_" + project.getName();
					}
				}
			}
			if (!StringUtils.isEmpty(customize)) {
				if (StringUtils.isEmpty(name)) {
					name += customize;
				} else {
					name += "_" + customize;
				}
			}
			// 后缀
			String suffixType = nameRule.getSuffixType();
			String suffix = null;
			if (NameRule.SuffixType.Millis.name().equalsIgnoreCase(suffixType)) {
				suffix = String.valueOf(System.currentTimeMillis());
			} else if (NameRule.SuffixType.RandomString.name().equalsIgnoreCase(suffixType)) {
				Long suffixLength = nameRule.getSuffixLength();
				if (null == suffixLength || 0L == suffixLength) {
					suffixLength = 10L;
				}
				suffix = getRandomString(suffixLength.intValue());
			} else if (NameRule.SuffixType.Number.name().equalsIgnoreCase(suffixType)) {
				Long nextNum = null == nameRule.getCurrentNum() ? 1L : nameRule.getCurrentNum();
				suffix = String.valueOf(nextNum);
				// 更新当前编号
				nameRule.setCurrentNum(nameRule.getCurrentNum() + 1L);
				String path = NameRule.class.getSimpleName() + "_" + id;
				try (AutoCloseLock lock = lockFactory.getACLock(path)) {
					Assert.isTrue(lock.acquire(20, TimeUnit.SECONDS), "请求超时");
					nameRuleRepository.update(nameRule);
				} catch (Exception e) {
					log.error("Remove name rule error:", e);
					return new GeneralResult<>(false, "生成失败");
				}
			}
			return new GeneralResult<>(true, name + "_" + suffix, "生成成功");
		} catch (Exception e) {
			log.error("Remove name rule error:", e);
			return new GeneralResult<>(false, "生成失败");
		}
	}

	// 生成指定length的随机字符串（A-Z，a-z，0-9）
	public static String getRandomString(int length) {
		String str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
		Random random = new SecureRandom();
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < length; i++) {
			int number = random.nextInt(str.length());
			sb.append(str.charAt(number));
		}
		return sb.toString();
	}

}
