package com.bocloud.cmp.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.curator.framework.recipes.cache.NodeCache;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.ButlerRepository;
import com.bocloud.cmp.domain.Butler;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.megatron.framework.registry.RegistryClient;

import lombok.extern.slf4j.Slf4j;

/**
 * 云服务供应商注册器
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年9月22日
 */
@Component
@Slf4j
public class ProviderRegister implements InitializingBean {

	private static NodeCache nodeCache;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	private final RegistryClient serviceClient;

	public void updateProvider(List<CloudVendor> providers) throws Exception {
		Map<String, String> map = convertProviders(providers);
		String data = JSONObject.parseObject(JSONObject.toJSONString(map)).toString();
		serviceClient.putButler(data);
	}

	@Autowired
	public ProviderRegister(RegistryClient serviceClient) {
		this.serviceClient = serviceClient;
	}

	/**
	 * 服务启动时将数据库中的云服务供应商信息注册到ZooKeeper中
	 *
	 * @throws Exception
	 */
	public void start() throws Exception {
		serviceClient.putButler(null);
		List<CloudVendor> providers = cloudVendorRepository.listAll();
		Map<String, String> map = convertProviders(providers);
		String data = JSONObject.parseObject(JSONObject.toJSONString(map)).toString();
		serviceClient.putButler(data);
		ButlerRepository.getInstance().update(data);
		// 处理ZooKeeper上的云服务供应商数据发生变化时的处理逻辑：更新内部的缓存
		nodeCache.getListenable().addListener(() -> {
			log.info("Cloud Service Provider data changed,Refresh them in cache!");
			ButlerRepository.getInstance().update(new String(nodeCache.getCurrentData().getData(), "utf-8"));
			log.info("Refresh Cloud Service Provider Success!");
		});
	}

	public Map<String, String> convertProviders(List<CloudVendor> providers) {
		Map<String, String> map = new HashMap<>();
		Butler butler = null;
		for (CloudVendor provider : providers) {
			JSONObject authentication = JSONObject.parseObject(provider.getAuthentication());
			String regions = authentication.getString("regions") == null ? "" : authentication.getString("regions");
			butler = new Butler();
			// 
			butler.setName(provider.getName());
			butler.setUuid(provider.getUuid());
			butler.setUsername(authentication.getString("account"));
			butler.setPassword(authentication.getString("password"));
			butler.setAccessKey(authentication.getString("accessKey"));
			butler.setSecretKey(authentication.getString("secretKey"));
			butler.setRegions(regions);
			butler.setTenant(authentication.getString("tenant"));
			butler.setUrl(authentication.getString("address"));
			butler.setCategory(provider.getType());
			butler.setDomainName(authentication.getString("domainName"));
			butler.setProjectName(authentication.getString("projectName"));
			butler.setVersion(provider.getVersion());
			butler.setRegionId(authentication.getString("regionId"));
			butler.setToken(authentication.getString("token"));
			butler.setProjectId(authentication.getString("projectId"));
			butler.setSubscriptionId(authentication.getString("subscriptionId"));
			butler.setClientId(authentication.getString("clientId"));
			butler.setVersion(authentication.getString("version"));
			butler.setProxyUri(authentication.getString("proxyUri"));
			butler.setLoginType(authentication.getInteger("loginType"));
			map.put(provider.getUuid(), JSON.toJSONString(butler));
		}
		return map;
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		nodeCache = new NodeCache(this.serviceClient.getClient().getClient(), serviceClient.getService().getButler(),
				false);
		nodeCache.start(true);
		this.start();
	}

}
